#include <stdio.h>
#include <string.h>

#include "addr.h"

int addr_len(const sockaddr_any *addr)
{
	switch (addr->any.sa_family) {
	case AF_INET:
		return sizeof(struct sockaddr_in);
	default:
		return 0;
	}
}

sockaddr_any *addr_parse_prefix(blob_t b, sockaddr_any *addr, uint8_t *prefix_len)
{
	memset(addr, 0, sizeof(*addr));

	if (blob_pull_inet_addr(&b, &addr->ipv4.sin_addr)) {
		addr->ipv4.sin_family = AF_INET;
		if (blob_pull_matching(&b, BLOB_STR("/"))) {
			if (prefix_len == NULL)
				return NULL;
			*prefix_len = blob_pull_uint(&b, 10);
		}
		if (b.len != 0)
			return NULL;
		return addr;
	}
	return NULL;
}

unsigned long addr_hash(const sockaddr_any *addr)
{
	switch (addr->any.sa_family) {
	case AF_INET:
		return htonl(addr->ipv4.sin_addr.s_addr);
	default:
		return 0;
	}
}

const char *addr_print(const sockaddr_any *addr)
{
	switch (addr->any.sa_family) {
	case AF_INET:
		return inet_ntoa(addr->ipv4.sin_addr);
	default:
		return "unknown";
	}
}

blob_t addr_get_hostaddr_blob(const sockaddr_any *addr)
{
	switch (addr->any.sa_family) {
	case AF_INET:
		return BLOB_BUF(&addr->ipv4.sin_addr);
	default:
		return BLOB_NULL;
	}
}

void addr_push_hostaddr(blob_t *b, const sockaddr_any *addr)
{
	char buf[64];
	blob_t f;
	unsigned int t;

	switch (addr->any.sa_family) {
	case AF_INET:
		t = ntohl(addr->ipv4.sin_addr.s_addr);
		f.ptr = buf;
		f.len = sprintf(buf, "%d.%d.%d.%d",
				(t      ) & 0xff, (t >>  8) & 0xff,
				(t >> 16) & 0xff, (t >> 24) & 0xff);
		break;
	default:
		return;
	}
	blob_push(b, f);
}

static int bitcmp(const uint8_t *a, const uint8_t *b, int len)
{
	int bytes, bits, mask, r;

	bytes = len / 8;
	bits  = len % 8;

	if (bytes != 0) {
		r = memcmp(a, b, bytes);
		if (r != 0)
			return r;
	}
	if (bits != 0) {
		mask = (0xff << (8 - bits)) & 0xff;
		return ((int) (a[bytes] & mask)) - ((int) (b[bytes] & mask));
	}
	return 0;
}

int addr_prefix_cmp(const sockaddr_any *a, const sockaddr_any *b, int prefix)
{
	if (a->any.sa_family != b->any.sa_family)
		return a->any.sa_family - b->any.sa_family;

	return bitcmp((const uint8_t *) &a->ipv4.sin_addr.s_addr,
		      (const uint8_t *) &b->ipv4.sin_addr.s_addr,
		      prefix);
}
